/***
 * OW2 FraSCAti explorer
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Christophe Demarey
 *
 * Contributor(s):
 * 
 */
package org.ow2.frascati.explorer.context;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.fractal.api.Component;
import org.objectweb.util.explorer.api.Context;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.core.naming.lib.DefaultEntry;
import org.ow2.frascati.explorer.util.fractal.FractalHelper;

/**
 * The FraSCAti Explorer Context for the AssemblyFactory.
 * (i.e. provides children of the AssemblyFactory) 
 * 
 * @author Christophe Demarey
 */
public class AssemblyFactoryContext 
  implements Context {

  // ==================================================================
  //
  // Internal state.
  //
  // ==================================================================
  
  private static final long serialVersionUID = 1988894854864485303L;
  
  /** The list of SCA composites loaded by the Assembly Factory. */
  private static List<Entry> composites = new ArrayList<Entry>();

  // ==================================================================
  //
  // Constructors.
  //
  // ==================================================================
  
  // ==================================================================
  //
  // Internal methods.
  //
  // ==================================================================

  // ==================================================================
  //
  // Public methods for interface Context.
  //
  // ==================================================================

  /**
   * Add a SCA composite to the list of child of the Assembly Factory.
   * Note : this implementation managed only one Factory.
   * 
   * @param composite A newly created SCA composite.
   */
  public static void addEntry(Component composite) {
    Entry entry = new DefaultEntry(FractalHelper.getComponentName(composite), composite);
    composites.add(entry);
  }

  /**
   * Get children of the Assembly Factory.
   * Note : this implementation managed only one Factory.
   * 
   * @see org.objectweb.util.explorer.api.Context#getEntries(Object)
   */
  public Entry[] getEntries(Object object) {
    return composites.toArray(new Entry[0]);
  }

}
