/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.implementation;

import org.eclipse.stp.sca.Implementation;
import org.eclipse.stp.sca.SCAImplementation;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.ow2.frascati.factory.core.dispatcher.Manager;

/**
 * Create SCA component instances implemented as SCA composite
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@org.objectweb.fractal.fraclet.annotations.Component
public class ScaCompositeImpl implements ScaImplementation {

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(ScaCompositeImpl.class.getCanonicalName());

  /**
   * The assembly factory dispatcher component required for recursive calls
   */

  @Requires(name = "loader")
  Manager dispatcher;

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.implementation.ScaImplementation#create(
   * org.eclipse.stp.sca.Implementation,
   * org.objectweb.fractal.api.type.ComponentType)
   */

  public Component create(Implementation implementation,
      ComponentType componentType) {
    
    //Retrieve composite file name 
    SCAImplementation ScaImpl = (SCAImplementation) implementation;
    String compositeName = ScaImpl.getName().getLocalPart();
    
    Component component = (org.objectweb.fractal.api.Component) dispatcher
    .loadComposite(compositeName);
    
    // TODO can only be used at runtime
    //if(component == null) throw new ImplementationException("Could not retrieve composite file :" + compositeName);

    return component;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.ow2.frascati.factory.core.instance.implementation.ScaImplementation#
   * getImplementationID()
   */

  public int getImplementationID() {
    return org.eclipse.stp.sca.ScaPackage.SCA_IMPLEMENTATION;
  }

}
