/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Nicolas Dolet
 * 
 * Contributor(s):
 */

package org.ow2.frascati.factory.core.instance.intent;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.ow2.frascati.factory.core.dispatcher.Manager;

/**
 * Abstract {@link ScaIntent} class for FraSCAti intent components
 * 
 * @author <a href="mailto:nicolas.dolet@inria.fr">Nicolas Dolet</a>
 * @version $Revision$
 */
@org.objectweb.fractal.fraclet.annotations.Component
public abstract class AbstractIntentPlugin
           implements ScaIntent {
  
  // Reference on the load interface provided by the Factory
  @Requires(name = "loader")
  Manager dispatcher;
  
  // Cache for loaded intents
  protected HashMap<String, Component> cache = new HashMap<String, Component>();
  
  //--------------------------------------------------------------------------
  // Protected methods
  // --------------------------------------------------------------------------

  /**
   * Get the business server interface names of a component
   * @param c a component
   * @return a list containing server interface names of c
   */
  protected List<String> getServerItfNames(Component c) {
    Object[] itfs = c.getFcInterfaces();
    List<String> ret = new ArrayList<String>();
    for(Object itf : itfs) {
      Interface i = (Interface) itf;
      String itfName = i.getFcItfName();
      if(!(itfName.endsWith("-controller") || itfName.equals("component"))) {
        InterfaceType type = (InterfaceType) i.getFcItfType();
        if(!type.isFcClientItf()) {
          ret.add(itfName);
        }
      }
    }
    return ret;
  }
  
  /**
   * Get the business client interface names of a component
   * @param c a component
   * @return a list containing client interface names of c
   */
  protected List<String> getClientItfNames(Component c) {
    Object[] itfs = c.getFcInterfaces();
    List<String> ret = new ArrayList<String>();
    for(Object itf : itfs) {
      Interface i = (Interface) itf;
      String itfName = i.getFcItfName();
      if(!(itfName.endsWith("-controller") || itfName.equals("component"))) {
        InterfaceType type = (InterfaceType) i.getFcItfType();
        if(type.isFcClientItf()) {
          ret.add(itfName);
        }
      }
    }
    return ret;
  }
  
  /**
   * Get a intent component from the cache, if present, else, load the
   * intent composite
   * @param intentName the {@link ScaIntent} composite name
   * @return the {@link Component} representing the intent
   */
  protected Component getFromCache(String intentName) {
    Component res = cache.get(intentName);
    if(res == null) {
      // Load this intent component
      res = (Component) dispatcher.loadComposite(intentName);
      // Put intent component into cache
      cache.put(intentName, res);
    }
    return res;
  }
}
