/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Nicolas Dolet
 * 
 * Contributor(s):
 */

package org.ow2.frascati.factory.core.instance.intent;

import java.util.List;

import javax.xml.namespace.QName;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.stp.sca.ComponentReference;
import org.eclipse.stp.sca.ScaPackage;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;
import org.ow2.frascati.tinfi.TinfiDomain;
import org.ow2.frascati.tinfi.control.intent.IntentHandler;
import org.ow2.frascati.tinfi.control.intent.SCAIntentController;

/**
 * {@link ScaIntent} implementation that set intents for client interfaces
 * 
 * @author <a href="mailto:nicolas.dolet@inria.fr">Nicolas Dolet</a>
 * @version $Revision$
 */
@org.objectweb.fractal.fraclet.annotations.Component
public class ReferenceIntent
     extends AbstractIntentPlugin {
  
  // --------------------------------------------------------------------------
  // Implementation of ScaIntent
  // --------------------------------------------------------------------------
  
  /**
   * @see org.ow2.frascati.factory.core.instance.intent.ScaIntent#getEObjectID()
   */
  public int getEObjectID() {
    return ScaPackage.COMPONENT_REFERENCE;
  }

  /**
   * @see org.ow2.frascati.factory.core.instance.intent.ScaIntent#setIntent(Object, EObject)
   */
  public void setIntent(Object o, EObject modelElement) throws IntentException {

    // Retrieve fractal component
    Component component = (Component) o;

    // Retrieve sca (model) reference
    ComponentReference componentReferenceElement = (ComponentReference) modelElement;

    List<QName> intents = componentReferenceElement.getRequires();

    for (QName name : intents) {

      // Get Intent name;
      String intentName = name.getLocalPart();

      // Try to retrieve intent reference from cache
      Component intentReference = getFromCache(intentName);

      try {
        // Stop Intent component
        Fractal.getLifeCycleController(intentReference).stopFc();

        // Get intent handler
        IntentHandler h = TinfiDomain.getService(intentReference,
            IntentHandler.class, "intent");

        // Get intent controller
        SCAIntentController ic = (SCAIntentController) component
            .getFcInterface(SCAIntentController.NAME);

        // Add handler on client interfaces
        List<String> clientItfNames = getClientItfNames(component);
        for (String itfName : clientItfNames) {
          ic.addFcIntentHandler(h, itfName);
        }

        // Start intent component
        Fractal.getLifeCycleController(intentReference).startFc();

      } catch (Exception e) {
        throw new IntentException("Intent : " + intentName
            + " can not be added to component "
            + componentReferenceElement.getName(), e);
      }

    }
  }
}
