/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Philippe Merle
 */

package org.ow2.frascati.factory.generate.implementation;

import org.eclipse.stp.sca.Implementation;
import org.eclipse.stp.sca.domainmodel.tuscany.SpringImplementation;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.ow2.frascati.factory.core.instance.implementation.ScaImplementation;

// import org.springframework.context.support.ClassPathXmlApplicationContext;


/**
 * Create FraSCAti component instances implemented with the Spring Framework.
 * 
 * @author Philippe Merle
 */

@Component
public class ScaSpringImpl implements ScaImplementation
{
  // ======================================================================
  //
  // Internal state.
  //
  // ======================================================================

  /**
   * Logger for this class.
   */
  protected java.util.logging.Logger log = java.util.logging.Logger
    .getLogger(this.getClass().getCanonicalName());

  /**
   * The Fractal generic factory used to create Fractal components wrapping Spring beans.
   */
  @Requires(name = "generic-factory")
  protected GenericFactory genericFactory;

  // ======================================================================
  //
  // Constructor.
  //
  // ======================================================================

  // ======================================================================
  //
  // Implementation of org.ow2.frascati.factory.instance.implementation.ScaImplementation
  //
  // ======================================================================

  /** (non-Javadoc)
   * @see ScaImplementation#getImplementationID()
   */
  public int getImplementationID ()
  {
    return org.eclipse.stp.sca.domainmodel.tuscany.TuscanyPackage.SPRING_IMPLEMENTATION;
  }

  /** (non-Javadoc)
   * @see ScaImplementation#create(Implementation,ComponentType)
   */
  public org.objectweb.fractal.api.Component create (
    final Implementation implementation,
    final ComponentType componentType
  ) {
    // Implementation is a Spring one.
    SpringImplementation springImplementation = (SpringImplementation) implementation;
    // Get the Spring location.
    String springLocation = springImplementation.getLocation();
    
    log.finer("Create an SCA component with the Spring implementation "
            + springLocation + " and the Fractal component type " + componentType.toString());

    // TODO: Must handle location as described in the SCA specification.
    // location could be a Jar file or a directory.
    // Currently, location is considered to refer a Spring XML file in the class path.

    org.objectweb.fractal.api.Component component = null;
    try {
      System.out.println("Generate composite for " + springLocation);

      // Create the Fractal component via the generic factory.
      component = genericFactory.newFcInstance(componentType, "scaComposite", null);
    } catch(InstantiationException ie) {
      // This should not happen!
      // TODO: Add an explicit exception to the 'create' method to report problems!
      throw new Error("Internal Fractal error!", ie);
    }

    // Return the Fractal component.
    return component;
  }
}
