/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.juliac;

import java.lang.reflect.Modifier;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Type;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.objectweb.fractal.juliac.proxy.AbstractProxyClassGenerator;
import org.objectweb.fractal.juliac.ucf.UnifiedClass;
import org.objectweb.fractal.juliac.ucf.UnifiedMethod;
import org.objectweb.fractal.juliac.visit.ClassSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.MethodSourceCodeVisitor;
import org.osoa.sca.RequestContext;
import org.osoa.sca.annotations.EndsConversation;
import org.ow2.frascati.tinfi.TinfiComponentCBInterface;
import org.ow2.frascati.tinfi.control.content.SCAContentController;
import org.ow2.frascati.tinfi.osoa.RequestContextImpl;

/**
 * This class generates the source code of component callback interface
 * implementations.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class CallBackInterfaceClassGenerator
extends AbstractProxyClassGenerator {
    
    public CallBackInterfaceClassGenerator() {
        super();
    }
    
    public CallBackInterfaceClassGenerator(
        InterfaceType it, MembraneDesc membraneDesc, boolean mergeable,
        Juliac jc ) {
        
        super(it,membraneDesc,mergeable,jc);
    }
    
    /**
     * Return true if some code must be generated by the current source file
     * generator.
     */
    public boolean match() {
        return true;
    }

    /**
     * Return the name of the suffix which is appended to generated class names.
     */
    public String getClassNameSuffix() {
        return "FcInItf";
    }

    @Override
    public void setMergeable( boolean mergeable ) {
        if(mergeable) {
            String msg = "This generator can not generate mergeable code";
            throw new IllegalArgumentException(msg);
        }
    }

    @Override
    public String getSuperClassName() {
        return TinfiComponentCBInterface.class.getName();
    }

    @Override
    public String[] getImplementedInterfaceNames() {
        return new String[]{it.getFcItfSignature()};
    }
    
    @Override
    public void generateConstructors( ClassSourceCodeVisitor cv, UnifiedClass proxycl ) {
        super.generateConstructors(cv, proxycl);
        
        // public constructor()
        MethodSourceCodeVisitor mv = cv.visitConstructor(Modifier.PUBLIC,null,null,null);
        mv.visitBegin();
        mv.visitEnd();

        // public constructor(Component,String,Type,boolean,Object)
        String[] parameters =
            new String[]{
                Component.class.getName()+" component",
                "String s",
                Type.class.getName()+" type",
                "boolean flag",
                "Object obj"
            };
        mv = cv.visitConstructor(Modifier.PUBLIC,null,parameters,null);
        mv.visitBegin();
        mv.visitln("    super(component,s,type,flag,obj);");
        mv.visitEnd();
    }
    
    @Override
    public void generateProxyMethodBodyBeforeCode(
        MethodSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym ) {
        super.generateProxyMethodBodyBeforeCode(mv, proxycl, proxym);

        // Create the request context
        mv.visit  ("    "+RequestContext.class.getName()+" rq = new ");
        mv.visit  (RequestContextImpl.class.getName());
        mv.visit  ("<"+it.getFcItfSignature()+">(\""+it.getFcItfName());
        mv.visitln("\","+it.getFcItfSignature()+".class,this);");
        
        // TODO the request context should be given a client proxy (not this which is a server one)
        
        /*
         * Push the request context on the thread local stack.
         * Initialize the thread local variable (can not be initialized in
         * ContentControllerMixin as we need a per-thread stack).
         */
        mv.visitln("    "+SCAContentController.class.getName()+" tla = getCC();");
        mv.visit  ("    java.util.Stack<"+RequestContext.class.getName());
        mv.visitln("> stack = tla.getRequestContextStack();");
        mv.visitln("    if( stack == null ) {");
        mv.visitln("      stack = new java.util.Stack<"+RequestContext.class.getName()+">();");
        mv.visitln("      tla.setRequestContextStack(stack);");
        mv.visitln("    }");
        mv.visitln("    stack.push(rq);");
        
        // Proceed with the call to the delegate
        mv.visitln("    "+it.getFcItfSignature()+" impl = ("+it.getFcItfSignature()+") getFcItfImpl();");
    }

    @Override
    public void generateProxyMethodBodyAfterCode(
        MethodSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym ) {
        super.generateProxyMethodBodyAfterCode(mv, proxycl, proxym);

        // Release the content instance
        mv.visit("    getCC().releaseFcContent(impl,");
        if( proxym.getAnnotation(EndsConversation.class) == null ) {
            mv.visit("false");
        }
        else {
            mv.visit("true");
        }
        mv.visitln(");");
        
        // Pop the request context
        mv.visitln("    stack.pop();");
    }

    public String getDelegatingInstance( UnifiedClass proxycl, UnifiedMethod proxym ) {
        return "impl";
    }
}
